/*!
 * quasar-ui-json-api-viewer v2.0.0-alpha.3
 * (c) 2021 Jeff Galbraith <jeff@quasar.dev>
 * Released under the MIT License.
 */

(function (global, factory) {
  typeof exports === 'object' && typeof module !== 'undefined' ? module.exports = factory(require('vue'), require('quasar'), require('prismjs')) :
  typeof define === 'function' && define.amd ? define(['vue', 'quasar', 'prismjs'], factory) :
  (global = typeof globalThis !== 'undefined' ? globalThis : global || self, global.jsonApiViewer = factory(global.Vue, global.Quasar));
}(this, (function (vue, quasar) { 'use strict';

  const NAME_PROP_COLOR = [
    'bg-orange-8',
    'bg-accent',
    'bg-secondary',
    'bg-warning'
  ];

  var JsonApiItem = vue.defineComponent({
    name: 'JsonApiItem',

    props: {
      name: {
        type: String,
        required: true
      },
      type: {
        type: String,
        default: 'props'
      },
      json: {
        type: Object,
        required: true
      },
      deprecatedColor: {
        type: String,
        default: 'yellow-8'
      },
      deprecatedBackground: {
        type: String,
        default: 'red-8'
      },
      removedColor: {
        type: String,
        default: 'yellow-8'
      },
      removedBackground: {
        type: String,
        default: 'red-8'
      }
    },

    setup (props, { slots }) {
      function highlight (str, lang) {
        if (lang === '') {
          lang = 'js';
        }
        else if (lang === 'vue' || lang === 'html') {
          lang = 'html';
        }

        if (Prism.languages[ lang ] !== undefined) {
          const code = Prism.highlight(str, Prism.languages[ lang ], lang);

          return '<pre class="q-markdown--code">'
            + `<code class="q-markdown--code__inner language-${ lang }">${ code }</code></pre>\n`
        }

        return ''
      }

      function getMethodName (title, json) {
        let name;

        name = ' [';

        if (json.returns) {
          name += json.returns.type;
        }
        else {
          name += 'void 0';
        }

        name += '] ';

        if (json.params === void 0) {
          name += title + ' ()';
        }
        else {
          name += title + ' (' + Object.keys(json.params).join(', ') + ')';
        }

        return name
      }

      function getEventName () {
        let name = '@' + props.name + ' => function';

        if (props.json.params === void 0) {
          return name + ' ()'
        }

        name += ' (' + Object.keys(props.json.params).join(', ') + ')';

        return name
      }

      function __renderSubitem (name, json, level = 0) {
        if (json.type === 'Function') {
          name = getMethodName(name, json);
        }
        return vue.h('div', {
          class: 'component-api__row component-api__row--bordered row'
        }, {
          default: () => [
            __renderName(name, NAME_PROP_COLOR[ level ]),
            __renderType(json),
            props.json.removedIn !== void 0 && __renderRemovedIn(json),
            props.json.removedIn === void 0 && props.json.deprecated !== void 0 && __renderDeprecated(json),
            props.json.removedIn === void 0 && props.json.deprecated === void 0 && __renderAddedIn(json),
            __renderRequired(json),
            __renderSync(json),
            __renderDefault(json),
            __renderApplicable(json),

            __renderDesc(json),
            __renderValues(json),
            __renderExamples(json),
            __renderParams(json, level + 1),
            __renderDefinitions(json, level + 1),
            __renderScope(json, level + 1),
            __renderReturns(json, level + 1)
          ]
        })
      }

      function __renderParams (json, level = 0) {
        if (json.params === void 0) return ''
        const keys = Object.keys(json.params);
        return vue.h('div', {
          class: 'component-api__row--item full-width'
        }, {
          default: () => [
            vue.h('div', {
              class: 'component-api__row--label'
            }, 'Parameter' + (Object.keys(json.params).length > 1 ? 's' : '')),
            vue.h('div', {
            }, {
              default: () => [
                keys.map(key => __renderSubitem(key, json.params[ key ], level))
              ]
            })
          ]
        })
      }

      function __renderDefinitions (json, level = 0) {
        if (json.definition === void 0) return ''
        const keys = Object.keys(json.definition);
        return vue.h('div', {
          class: 'component-api__row--item full-width'
        }, {
          default: () => [
            vue.h('div', {
              class: 'component-api__row--label'
            }, 'Definition' + (Object.keys(json.definition).length > 1 ? 's' : '')),
            vue.h('div', {
            }, {
              default: () => [
                keys.map(key => __renderSubitem(key, json.definition[ key ], level))
              ]
            })
          ]
        })
      }

      function __renderScope (json, level = 0) {
        if (json.scope === void 0) return ''
        const keys = Object.keys(json.scope);
        return vue.h('div', {
          class: 'component-api__row--item full-width'
        }, {
          default: () => [
            vue.h('div', {
              class: 'component-api__row--label'
            }, 'Scope'),
            vue.h('div', {
            }, {
              default: () => [
                keys.map(key => __renderSubitem(key, json.scope[ key ], level))
              ]
            })
          ]
        })
      }

      function __renderReturns (json, level = 0) {
        if (json.returns === void 0) return ''
        return vue.h('div', {
          class: 'component-api__row--item full-width'
        }, {
          default: () => [
            vue.h('div', {
              class: 'component-api__row--label'
            }, 'Returns'),
            vue.h('div', {
            }, {
              default: () => [
                json.returns === null && 'null',
                json.returns !== null && __renderSubitem(void 0, json.returns, level)
              ]
            })
          ]
        })
      }

      function __renderValues (json) {
        if (json.values === void 0 || json.values.length <= 0) return ''
        return vue.h('div', {
          class: 'component-api__row--item col-auto'
        }, {
          default: () => [
            vue.h('div', {
              class: 'component-api__row--label'
            }, 'Value' + (json.values.length > 1 ? 's' : '')),
            vue.h('div', {
              class: 'component-api__row--values'
            }, json.values.join(', '))
          ]
        })
      }

      function __renderExample (example) {
        const inner = highlight(example, 'js');

        return vue.h('div', {
          class: 'component-api__row--example',
          innerHtml: inner
        })
      }

      function __renderExamples (json) {
        if (json.examples === void 0 || json.examples.length <= 0) return ''
        return vue.h('div', {
          class: 'component-api__row--item col-auto'
        }, {
          default: () => [
            vue.h('div', {
              class: 'component-api__row--label'
            }, 'Example' + (json.examples.length > 1 ? 's' : '')),
            vue.h('div', {
              class: 'component-api__row--value'
            }, {
              default: () => [
                json.examples.map((example, index) => __renderExample(example))
              ]
            })
          ]
        })
      }

      function __renderDesc (json) {
        if (json.desc === void 0) return
        // const inner = highlight(json.desc, 'md')

        return vue.h('div', {
          class: 'component-api__row--item full-width'
        }, {
          default: () => [
            vue.h('div', {
              class: 'component-api__row--label'
            }, 'Description'),
            vue.h('div', {
              class: 'component-api__row--value'
            }, {
              default: () => [
                vue.h('div', {
                  // innerHTML: inner
                }, json.desc)
              ]
            })
          ]
        })
      }

      function __renderSync (json) {
        if (json.sync === void 0) return ''
        return vue.h('div', {
          class: 'component-api__row--item col-xs-12 col-sm-4'
        }, {
          default: () => [
            vue.h('div', {
              class: 'component-api__row--label'
            }, 'Sync'),
            vue.h('div', {
              class: 'component-api__row--value'
            }, {
              default: () => [
                vue.h('div', json.sync)
              ]
            })
          ]
        })
      }

      function __renderRequired (json) {
        if (json.required === void 0) return ''
        return vue.h('div', {
          class: 'component-api__row--item col-xs-12 col-sm-4'
        }, {
          default: () => [
            vue.h('div', {
              class: 'component-api__row--label'
            }, 'Required'),
            vue.h('div', {
              class: 'component-api__row--value'
            }, {
              default: () => [
                vue.h('div', json.required)
              ]
            })
          ]
        })
      }

      function __renderApplicable (json) {
        if (json.applicable === void 0) return ''
        return vue.h('div', {
          class: 'component-api__row--item col-xs-12 col-sm-4'
        }, {
          default: () => [
            vue.h('div', {
              class: 'component-api__row--label'
            }, 'Applicable'),
            vue.h('div', {
              class: 'component-api__row--value'
            }, {
              default: () => [
                vue.h('div', json.applicable.join(', '))
              ]
            })
          ]
        })
      }

      function __renderDefault (json) {
        if (json.default === void 0) return ''
        return vue.h('div', {
          class: 'component-api__row--item col-xs-12 col-sm-4'
        }, {
          default: () => [
            vue.h('div', {
              class: 'component-api__row--label'
            }, 'Default'),
            vue.h('div', {
              class: 'component-api__row--value'
            }, {
              default: () => [
                vue.h('div', json.default)
              ]
            })
          ]
        })
      }

      function __renderRemovedIn (json) {
        if (json.removedIn === void 0) return ''
        return vue.h('div', {
          class: 'component-api__row--item col-xs-12 col-sm-4'
        }, {
          default: () => [
            vue.h('div', {
              class: 'component-api__row--label'
            }, {
              default: () => [
                vue.h('span', {
                  class: 'rounded-borders text-' + props.removedColor + ' bg-' + props.removedBackground
                }, 'Removed in')
              ]
            }),
            vue.h('div', {
              class: 'component-api__row--value'
            }, {
              default: () => [
                vue.h('div', {
                  default: () => [
                    vue.h('span', {
                      class: 'rounded-borders text-' + props.deprecatedColor + ' bg-' + props.deprecatedBackground
                    }, json.removedIn)
                  ]
                })
              ]
            })
          ]
        })
      }

      function __renderDeprecated (json) {
        if (json.deprecated === void 0) return ''
        return vue.h('div', {
          class: 'component-api__row--item col-xs-12 col-sm-4'
        }, {
          default: () => [
            vue.h('div', {
              class: 'component-api__row--label'
            }, {
              default: () => [
                vue.h('span', {
                  class: 'rounded-borders text-' + props.deprecatedColor + ' bg-' + props.deprecatedBackground
                }, 'Deprecated')
              ]
            }),
            vue.h('div', {
              class: 'component-api__row--value'
            }, {
              default: () => [
                vue.h('span', {
                  class: 'rounded-borders text-' + props.deprecatedColor + ' bg-' + props.deprecatedBackground
                }, json.deprecated)
              ]
            })
          ]
        })
      }

      function __renderAddedIn (json) {
        if (json.addedIn === void 0) return ''
        return vue.h('div', {
          class: 'component-api__row--item col-xs-12 col-sm-4'
        }, {
          default: () => [
            vue.h('div', {
              class: 'component-api__row--label'
            }, 'Added in'),
            vue.h('div', {
              class: 'component-api__row--value'
            }, {
              default: () => [
                vue.h('div', json.addedIn)
              ]
            })
          ]
        })
      }

      function __renderType (json) {
        if (json.type === void 0) return ''
        const type = Array.isArray(json.type) ? json.type.join(' | ') : json.type;
        return vue.h('div', {
          class: 'component-api__row--item col-xs-12 col-sm-4'
        }, {
          default: () => [
            vue.h('div', {
              class: 'component-api__row--label'
            }, 'Type'),
            vue.h('div', {
              class: 'component-api__row--value'
            }, {
              default: () => [
                vue.h('div', type)
              ]
            })
          ]
        })
      }

      function __renderName (name, color) {
        if (name === void 0) return ''
        return vue.h('div', {
          class: 'component-api__row--item col-grow'
        }, {
          default: () => [
            vue.h('div', {
              class: 'component-api__row--label'
            }, 'Name'),
            vue.h('div', {
              class: 'component-api__row--value'
            }, {
              default: () => [
                vue.h(quasar.QBadge, {
                  class: `property-name ${ color }`
                }, {
                  default: () => name
                })
              ]
            })
          ]
        })
      }

      function __render () {
        let name = props.name;
        if (props.type === 'methods') {
          name = getMethodName(name, props.json);
        }
        else if (props.type === 'events') {
          name = getEventName(name, props.json);
        }
        const level = 0;
        return vue.h('div', {
          class: 'row full-width'
        }, {
          default: () => [
            __renderName(name, NAME_PROP_COLOR[ level ]),
            __renderType(props.json),
            props.json.removedIn === void 0 && __renderDeprecated(props.json),
            props.json.deprecated === void 0 && __renderAddedIn(props.json),
            __renderRemovedIn(props.json),
            __renderRequired(props.json),
            __renderSync(props.json),
            __renderDefault(props.json),
            __renderApplicable(props.json),

            __renderDesc(props.json),
            __renderValues(props.json),
            __renderExamples(props.json),
            __renderParams(props.json, level + 1),
            __renderDefinitions(props.json, level + 1),
            __renderScope(props.json, level + 1),
            __renderReturns(props.json, level + 1)
          ]
        })
      }

      function render () {
        return __render()
      }

      return () => render()
    }
  });

  var JsonApiList = vue.defineComponent({
    name: 'JsonApiList',

    props: {
      name: {
        type: String,
        required: true
      },
      json: {
        type: Object,
        required: true
      }
    },

    setup (props) {
      const __headings = vue.computed(() => {
        return Object.keys(props.json)
      });

      function __renderItems () {
        return [...__headings.value.map(heading => vue.h(quasar.QItem, {
          key: props.name + '-' + heading,
          class: 'component-api__list-item'
        }, {
          default: () => [
            vue.h(JsonApiItem, {
              name: heading,
              type: props.name,
              json: props.json[ heading ]
            })
          ]
        }))]
      }

      function __render () {
        const content = Object.keys(props.json).length !== 0
          ? __renderItems()
          : vue.h('div', { staticClass: 'q-pa-md text-grey-7' }, 'No matching entries found. Please refine the filter.');

        return vue.h(quasar.QList, {
          class: 'component-api__list',
          separator: true
        }, {
          default: () => content
        })
      }

      function render () {
        return __render()
      }

      return () => render()
    }
  });

  function copyToClipboard (text) {
    const textArea = document.createElement('textarea');
    textArea.className = 'fixed-top';
    textArea.value = text;
    document.body.appendChild(textArea);
    textArea.focus();
    textArea.select();

    document.execCommand('copy');
    document.body.removeChild(textArea);
  }

  function copyHeading (id) {
    const text = window.location.origin + window.location.pathname + '#' + id;
    const el = document.getElementById(id);

    if (el) {
      el.id = '';
    }

    if ('replaceState' in history) {
      history.replaceState(history.state, '', `${ location.pathname }#${ id }`);
    }
    else {
      window.location.hash = '#' + id;
    }

    if (el) {
      setTimeout(() => {
        el.id = id;
      }, 300);
    }

    copyToClipboard(text);

    quasar.Notify.create({
      message: 'Anchor has been copied to clipboard.',
      position: 'top',
      color: 'white',
      textColor: 'primary',
      actions: [{ icon: 'cancel', color: 'red-8', dense: true, round: true }],
      timeout: 2000
    });
  }

  // eslint-disable-next-line
  const specialRE = /[\s·/_\\,:;\.\(\)\[\]]+/g;
  const andRE = /&/g;
  const nonWordRE = /[^\w-]+/g;
  const multipleDashRE = /--+/g;

  function slugify (str) {
    return String(str).toLowerCase()
      .replace(specialRE, '-')
      .replace(andRE, '-and-')
      .replace(nonWordRE, '')
      .replace(multipleDashRE, '-')
  }

  var version$1 = "2.0.0-alpha.3";

  /* eslint-disable comma-dangle */

  var Component = vue.defineComponent({
    name: 'JsonApiViewer',

    directives: { ClosePopup: quasar.ClosePopup },

    props: {
      title: {
        type: String,
        required: true
      },
      json: {
        type: Object,
        required: true
      },
      type: {
        type: String,
        default: 'Vue Component'
      },
      noMenu: Boolean,
      startingTab: String,
      startingInnerTab: String,
      noFooter: Boolean
    },

    setup (props, { slots }) {
      const
        // $q = useQuasar(),
        ready = vue.ref(false),
        currentTab = vue.ref('props'),
        currentInnerTab = vue.ref('model'),
        // api = ref(null),
        filteredApi = vue.ref(null),
        filter = vue.ref(null),
        tabs = vue.ref([]),
        tabCount = vue.reactive({}),
        innerTabCount = vue.reactive({}),
        innerTabContent = vue.reactive({}),
        borderColor = vue.ref('lightblue'),
        separatorColor = vue.ref('light-blue-2'),
        showDeprecated = vue.ref(false),
        showRemoved = vue.ref(false),
        inputRef = vue.ref(null),
        vm = vue.getCurrentInstance(),
        $q = vm.ctx.$q;

      console.log('vm', vm);

      const __slugifiedTitle = vue.computed(() => slugify(props.title));

      // const __headings = computed(() => tabs.value)

      vue.watch(() => filter.value, () => {
        __parseJson(props.json);
      });

      vue.watch(() => showDeprecated.value, () => {
        __parseJson(props.json);
      });

      vue.watch(() => showRemoved.value, () => {
        __parseJson(props.json);
      });

      vue.onBeforeMount(() => {
        currentTab.value = props.startingTab || currentTab.value;
        currentInnerTab.value = props.startingInnerTab || currentInnerTab.value;
        __parseJson(props.json);
      });

      function __parseJson (json) {
        if (json === void 0) {
          // no api
          ready.value = true;
          return
        }

        // deep copy object
        const api = quasar.extend(true, {}, json);

        if (api.type !== void 0) {
          delete api.type;
        }

        if (api.meta) {
          delete api.meta;
        }

        __resetFiltered();

        // deal with "props"
        if (api.props !== void 0) {
          // get sorted keys
          const propKeys = Object.keys(api.props).sort();

          // loop through keys to get inner tab content
          for (let j = 0; j < propKeys.length; ++j) {
            const key = propKeys[ j ];
            const p = api.props[ key ];
            if (p.deprecated !== void 0 && props.showDeprecated !== true) {
              delete api.props[ key ];
              continue
            }
            if (p.removedIn !== void 0 && props.showRemoved !== true) {
              delete api.props[ key ];
              continue
            }
            if (innerTabContent[ p.category ] === void 0) {
              innerTabContent[ p.category ] = {};
            }
            if (__filterKey(key)) {
              innerTabContent[ p.category ][ key ] = p;
            }
            else {
              const apiProps = __filterContent(p);
              if (Object.keys(apiProps).length > 0) {
                innerTabContent[ p.category ][ key ] = apiProps;
              }
              else {
                delete api.props[ key ];
              }
            }
          }

          __deleteKeys(innerTabCount);
          const innerKeys = Object.keys(innerTabContent);
          for (let k = 0; k < innerKeys.length; ++k) {
            const innerPropKey = innerKeys[ k ];
            innerTabCount[ innerPropKey ] = Object.keys(innerTabContent[ innerPropKey ]).length;
          }
        }

        __filterTopLevel(api);

        // get a count of items within each top-level tab menu
        const keys = Object.keys(api);
        for (let i = 0; i < keys.length; ++i) {
          const type = keys[ i ];
          tabs.value[ type ] = Object.keys(api[ type ]).length;
        }

        filteredApi.value = api;

        ready.value = true;
      }

      function __filterTopLevel (api) {
        const keys = Object.keys(api);
        for (let i = 0; i < keys.length; ++i) {
          const type = keys[ i ];
          if (type !== 'props') {
            // loop through inner content
            const propKeys = Object.keys(api[ type ]);
            for (let l = 0; l < propKeys.length; ++l) {
              const key = propKeys[ l ];
              const p = api[ type ][ key ];
              if (p.deprecated !== void 0 && props.showDeprecated !== true) {
                delete api[ type ][ key ];
              }
              if (p.removedIn !== void 0 && props.showRemoved !== true) {
                delete api[ type ][ key ];
              }
              else if (__filterKey(key) !== true) {
                const apiProps = __filterContent(p);
                if (Object.keys(apiProps).length === 0) {
                  delete api[ type ][ key ];
                }
              }
            }
          }
        }
      }

      function __filterContent (api) {
        if (filter.value === null) {
          return api
        }

        let found = false;
        const keys = Object.keys(api);
        for (let i = 0; i < keys.length; ++i) {
          const prop = api[ keys[ i ] ];
          const type = Object.prototype.toString.call(prop);
          if (type === '[object Array]') {
            if (__filterArray(prop)) {
              found = true;
              break
            }
          }
          else if (type === '[object String]' || type === '[object Number]') {
            if (__filterString(prop)) {
              found = true;
              break
            }
          }
        }

        if (found !== true) {
          return {}
        }

        return api
      }

      // tests the passed string against the filter
      function __filterString (str) {
        if (filter.value) {
          return String(str).toLowerCase().indexOf(filter.value.toLowerCase()) >= 0
        }
      }

      function __filterKey (str) {
        return __filterString(str)
      }

      function __filterArray (arr) {
        for (let i = 0; i < arr.length; ++i) {
          if (__filterString(arr[ i ]) === true) {
            return true
          }
        }
        return false
      }

      function __resetFiltered () {
        __deleteKeys(innerTabContent);
        __deleteKeys(innerTabCount);
        __deleteKeys(tabCount);
        tabs.value = [];
      }

      function __deleteKeys (obj) {
        Object.keys(obj).forEach(key => {
          delete obj[ key ];
        });
      }

      function __onFilter () {
        if (filter.value !== null) {
          filter.value = null;
        }

        inputRef.value.focus();
      }

      function __renderTitle () {
        return vue.h(quasar.QToolbarTitle, {
          class: props.noAnchor !== true ? 'example-title' : '',
          onClick: copyHeading
        }, {
          default: () => vue.h('span', {
            class: 'ellipsis'
          }, props.title)
        })
      }

      function __renderFilter () {
        return vue.h(quasar.QInput, {
          ref: inputRef,
          class: 'q-mx-sm',
          // style: {
          //   minWidth: '150px'
          // },
          placeholder: 'Filter...',
          modelValue: filter.value,
          inputClass: 'text-right',
          dense: true,
          borderless: true,
          'onUpdate:modelValue': v => {
            filter.value = v;
          }
        },
        {
          append: () => vue.h(quasar.QIcon, {
            class: 'cursor-pointer',
            name: filter.value !== null ? 'clear' : 'search',
            onClick: __onFilter
          })
        })
      }

      function __renderToolbar () {
        return vue.h(quasar.QToolbar, {}, {
          default: () => [
            __renderTitle(),
            vue.h('div', {
              class: 'q-ml-md col-auto text-grey text-caption'
            }, {
              default: () => [
                $q.screen.gt.xs && props.type
              ]
            }),
            $q.screen.width >= 385 && __renderFilter(),
            props.noMenu !== true && __renderMenu()
            // __renderMenu()
          ]
        })
      }

      function __renderMenu () {
        return vue.h(quasar.QIcon, {
          class: 'cursor-pointer text-grey',
          name: 'menu',
          size: 'md'
        }, {
          default: () => [
            vue.h(quasar.QMenu, {
              default: () => [
                vue.h('div', {
                  class: 'row no-wrap q-pa-md'
                }, {
                  default: () => [
                    vue.h('div', {
                      class: 'column'
                    }, {
                      default: () => [
                        vue.h(quasar.QCheckbox, {
                          modelValue: showDeprecated.value,
                          label: 'Show deprecated',
                          // directives: [{
                          //   name: 'close-popup'
                          // }],
                          'onUpdate:modelValue': val => {
                            showDeprecated.value = val;
                          }
                        }),
                        vue.h(quasar.QCheckbox, {
                          modelValue: showRemoved.value,
                          label: 'Show removed',
                          // directives: [{
                          //   name: 'close-popup'
                          // }],
                          'onUpdate:modelValue': val => {
                            showRemoved.value = val;
                          }
                        })
                      ]
                    })
                  ]
                })
              ]
            })
          ]
        })
      }

      function __renderTabs () {
        return vue.h('div', {
          class: 'row justify-between items-center no-wrap' + (!$q.dark.isActive ? ' bg-white text-grey-7' : '')
        }, {
          default: () => [
            vue.h(quasar.QTabs, {
              class: 'col-grow text-caption' + (!$q.dark.isActive ? ' bg-white text-grey-7' : ''),
              modelValue: currentTab.value,
              dense: true,
              activeColor: $q.dark.isActive ? 'yellow' : 'primary',
              indicatorColor: $q.dark.isActive ? 'yellow' : 'primary',
              align: 'left',
              narrowIndicator: true,
              'onUpdate:modelValue': v => {
                currentTab.value = v;
              }
            }, {
              default: () => __renderTab(tabs.value)
            })
          ]
        })
      }

      function __renderTab (tab) {
        return [
          ...Object.keys(tab).map(propKey => vue.h(quasar.QTab, {
            key: propKey + '-tab',
            name: propKey,
            style: {
              paddingRight: 0
            }
          }, {
            default: () => [
              __renderTabSlot(propKey, tab[ propKey ])
            ]
          }))
        ]
      }

      function __renderTabSlot (label, count, stretch) {
        console.log('label', label, count, stretch);
        return vue.h('div', {
          class: 'row no-wrap items-center self-stretch q-pr-sm' + (stretch ? ' justify-between' : ''),
          style: {
            minWidth: stretch === true ? '120px' : void 0
          }
        }, {
          default: () => [
            vue.h('span', {
              class: 'q-mr-xs text-capitalize text-weight-medium'
            }, label),
            count > 0 && vue.h(quasar.QBadge, null, { default: () => count })
          ]
        })
      }

      function __renderTabPanels () {
        return vue.h(quasar.QTabPanels, {
          modelValue: currentTab.value,
          animated: true,
          'onUpdate:modelValue': v => {
            currentTab.value = v;
          }
        }, {
          default: () => __renderTabPanel()
        })
      }

      function __renderTabPanel () {
        console.log('filteredApi', filteredApi);
        return [...Object.keys(tabs.value)
          .map(propKey => vue.h(quasar.QTabPanel, {
            key: propKey + '-panel',
            class: 'q-pa-none',
            name: propKey
          }, {
            default: () => [
              propKey === 'props' && __renderInnerTabs(propKey, filteredApi.value[ propKey ]),
              propKey !== 'props' && __renderApiList(propKey, filteredApi.value[ propKey ])
            ]
          }))
        ]
      }

      function __renderApiList (name, api) {
        return vue.h('div', {
          class: 'component-api__container'
        }, {
          default: () => [
            vue.h(JsonApiList, {
              name: name,
              json: api
            })
          ]
        })
      }

      function __renderInnerTabs (name, api) {
        return vue.h('div', {
          class: 'fit row'
        }, {
          default: () => [
            vue.h('div', {
              class: 'col-auto row no-wrap q-py-lg' + (!$q.dark.isActive ? ' bg-white text-grey-7' : '')
            }, {
              default: () => [
                vue.h(quasar.QTabs, {
                  class: 'text-caption' + (!$q.dark.isActive ? ' bg-white text-grey-7' : ''),
                  modelValue: currentInnerTab.value,
                  dense: true,
                  vertical: true,
                  activeColor: $q.dark.isActive ? 'yellow' : 'primary',
                  indicatorColor: $q.dark.isActive ? 'yellow' : 'primary',
                  align: 'left',
                  narrowIndicator: true,
                  'onUpdate:modelValue': v => {
                    currentInnerTab.value = v;
                  }
                }, {
                  default: () => [
                    ...Object.keys(innerTabCount)
                      .map(propKey => vue.h(quasar.QTab, {
                        key: propKey + '-inner-tab',
                        class: 'col-grow',
                        name: propKey
                      },
                      {
                        default: () => [
                          __renderTabSlot(propKey, innerTabCount[ propKey ], true)
                        ]
                      }
                      ))
                  ]
                })
              ]
            }),
            vue.h(quasar.QSeparator, {
              vertical: true,
              color: separatorColor.value,
              style: {
                minHeight: '600px'
              }
            }),
            __renderInnerTabPanels()
          ]
        })
      }

      function __renderInnerTabPanels () {
        return vue.h(quasar.QTabPanels, {
          class: 'col',
          modelValue: currentInnerTab.value,
          animated: true,
          transitionPrev: 'slide-down',
          transitionNext: 'slide-up',
          'onUpdate:modelValue': v => {
            currentInnerTab.value = v;
          }
        }, {
          default: () => [
            __renderInnerTabPanel()
          ]
        })
      }

      function __renderInnerTabPanel () {
        return [...Object.keys(innerTabContent)
          .map(propKey => vue.h(quasar.QTabPanel, {
            key: propKey + '-inner-panel',
            class: 'q-pa-none',
            name: propKey
          }, {
            default: () => [
              __renderApiList(propKey, innerTabContent[ propKey ])
            ]
          }))
        ]
      }

      function __renderFooter () {
        const slot = slots.footer;
        if (props.noFooter === true) return
        return vue.h('div', {
          class: 'component-api__footer row justify-between items-center'
        }, {
          default: () => [
            slot || vue.h('div', 'Quasar JSON API Viewer v' + version$1 + ' - Created and maintained by Jeff Galbraith (@hawkeye64)')
          ]
        })
      }

      function __renderCard () {
        return vue.h(quasar.QCard, {
          class: 'no-shadow',
          flat: true,
          bordered: true,
          style: {
            border: `${ borderColor.value } 1px solid`
          }
        }, {
          default: () => [
            __renderToolbar(),
            vue.h(quasar.QSeparator, {
              color: separatorColor.value
            }),
            __renderTabs(),
            vue.h(quasar.QSpace),
            vue.h(quasar.QSeparator, {
              color: separatorColor.value
            }),
            __renderTabPanels(),
            __renderFooter()
          ]
        })
      }

      function __renderSection () {
        return vue.h('section', {
          id: __slugifiedTitle.value,
          class: 'q-pa-md overflow-auto'
        }, __renderCard())
      }

      function __render () {
        return vue.h('div', __renderSection())
      }

      function renderJson () {
        return __render()
        // if (ready.value === true) {
        //   return __render()
        // }
        // return void 0
      }

      return () => renderJson()
    }
  });

  const version = '2.0.0-alpha.3';

  function install (app) {
    app.component(Component.name, Component);
  }

  var VuePlugin = /*#__PURE__*/Object.freeze({
    __proto__: null,
    version: version,
    Component: Component,
    install: install
  });

  return VuePlugin;

})));
